#include <stdio.h>
#include <stdlib.h>

/* Initialize this array with a single 1 and the rest zeros.  Observe what
 * happens to the executable size when it is changed to contain all zeros!
 */
char big_global[1L << 24] = { 1, };  /* 16 MB */
char huge_global[1L << 30]; /* 1 GB */

const char ro_global[4096] = { 1, };

int global = 4;
int zero = 0;

int my_function(void) { return 0; }

int main()
{
	void *h1, *h2, *h3, *h4;
	int local = 0;

	h1 = malloc(1L << 28); /* 256 MB */
	h2 = malloc(1L << 8);  /* 256  B */
	h3 = malloc(1L << 30); /*   1 GB */
	h4 = malloc(1L << 5);  /*  32  B */

#define PADDR(x, sz) printf("%12s = %012lx - %012lx\n",     \
                            #x, (unsigned long)(x),         \
                            (unsigned long)(x) + (sz) - 1)

	/* No way to get the size of a function in C; just pretend it's 1 */

	/* Text section: statically allocated, read-only, executable */
	PADDR(printf, 1);
	PADDR(malloc, 1);
	PADDR(my_function, 1);
	PADDR(main, 1);

	/* Statically allocated, read-only */
	PADDR(ro_global, sizeof(ro_global));

	/* Statically allocated, read/write */
	PADDR(&global, sizeof(global));
	PADDR(big_global, sizeof(big_global));

	/* Statically allocated and zero-initialized (BSS) */
	PADDR(&zero, sizeof(zero));
	PADDR(huge_global, sizeof(huge_global));

	/* Heap-allocated */
	PADDR(h2, 1L << 8);
	PADDR(h4, 1L << 5);
	PADDR(h3, 1L << 30);
	PADDR(h1, 1L << 28);

	/* Stack-allocated */
	PADDR(&local, sizeof(local));

	return 0;
}
